/**
 * \file
 * 
 * Private unstable APIs.
 *
 * WARNING: The declarations and behavior of functions in this header are NOT STABLE and can be modified or removed at
 * any time.
 *
 */


#ifndef __MONO_UTILS_MONO_PRIVATE_UNSTABLE_H__
#define __MONO_UTILS_MONO_PRIVATE_UNSTABLE_H__

#include <mono/utils/mono-publib.h>



#endif /*__MONO_UTILS_MONO_PRIVATE_UNSTABLE_H__*/
